@@ -217,6 +217,12 @@ void EditSelection::finalizeSelection() {
         this->snappedBounds.x = this->x + ox;
         this->snappedBounds.y = this->y + oy;
         v = this->contents->getSourceView();
+
+        PageRef page = v->getPage();
+        Layer* layer = page->getSelectedLayer();
+        // Create an Undo action to compensate - avoids Segfault/Freeze if the user presses undo after this happened
+        this->contents->updateContent(this->getRect(), this->snappedBounds, this->rotation, this->aspectRatio, layer,
+                                      page, this->undo, CURSOR_SELECTION_MOVE);
     }
 
 
@@ -224,8 +230,7 @@ void EditSelection::finalizeSelection() {
 
     PageRef page = this->view->getPage();
     Layer* layer = page->getSelectedLayer();
-    this->contents->finalizeSelection(this->getRect(), this->snappedBounds, this->aspectRatio, layer, page, this->view,
-                                      this->undo);
+    this->contents->finalizeSelection(this->getRect(), this->snappedBounds, this->aspectRatio, layer);
 
 
     // Calculate new clip region delta due to rotation:
@@ -462,7 +467,7 @@ void EditSelection::mouseUp() {
     this->sourceLayer = layer;
 
     this->contents->updateContent(this->getRect(), this->snappedBounds, this->rotation, this->aspectRatio, layer, page,
-                                  this->view, this->undo, this->mouseDownType);
+                                  this->undo, this->mouseDownType);
 
     this->mouseDownType = CURSOR_SELECTION_NONE;
 
@@ -627,15 +632,17 @@ void EditSelection::mouseMove(double mouseX, double mouseY, bool alt) {
 
     this->view->getXournal()->repaintSelection();
 
-    XojPageView* v = getPageViewUnderCursor();
+    if (this->mouseDownType == CURSOR_SELECTION_MOVE) {
+        XojPageView* v = getPageViewUnderCursor();
 
-    if (v && v != this->view) {
-        XournalView* xournal = this->view->getXournal();
-        const auto pageNr = xournal->getControl()->getDocument()->indexOf(v->getPage());
+        if (v && v != this->view) {
+            XournalView* xournal = this->view->getXournal();
+            const auto pageNr = xournal->getControl()->getDocument()->indexOf(v->getPage());
 
-        xournal->pageSelected(pageNr);
+            xournal->pageSelected(pageNr);
 
-        translateToView(v);
+            translateToView(v);
+        }
     }
 }
 
@@ -763,7 +770,19 @@ void EditSelection::moveSelection(double dx, double dy, bool addMoveUndo) {
     updateMatrix();
 
     if (addMoveUndo) {
-        this->contents->addMoveUndo(this->undo, dx, dy);
+        XojPageView* v = getPageViewUnderCursor();
+
+        if (v && v != this->view) {
+            XournalView* xournal = this->view->getXournal();
+            const auto pageNr = xournal->getControl()->getDocument()->indexOf(v->getPage());
+
+            xournal->pageSelected(pageNr);
+
+            translateToView(v);
+        }
+        this->contents->updateContent(this->getRect(), this->snappedBounds, this->rotation, this->aspectRatio,
+                                      this->view->getPage()->getSelectedLayer(), this->view->getPage(), this->undo,
+                                      CURSOR_SELECTION_MOVE);
     }
 
     this->view->getXournal()->repaintSelection();
@@ -796,25 +815,40 @@ bool EditSelection::handleEdgePan(EditSelection* self) {
     const double zoom = self->view->getXournal()->getZoom();
 
     // Helper function to compute scroll amount for a single dimension, based on visible region and selection bbox
-    const auto computeScrollAmt = [&](double visMin, double visLen, double bboxMin, double bboxLen,
-                                      double layoutSize) -> double {
+    const auto computeScrollAmt = [&](double visMin, double visLen, double bboxMin, double bboxLen, double layoutSize,
+                                      double relMousePos) -> double {
         const bool belowMin = bboxMin < visMin;
         const bool aboveMax = bboxMin + bboxLen > visMin + visLen;
         const double visMax = visMin + visLen;
         const double bboxMax = bboxMin + bboxLen;
 
+        const bool isLargeSelection = bboxLen > visLen;
+        const auto centerVis = (visMin + visLen / 2);
+        const auto mouseDiff = (bboxMin + relMousePos * zoom - centerVis);
+
         // Scroll amount multiplier
         double mult = 0.0;
 
-        // Calculate bonus scroll amount due to proportion of selection out of view.
         const double maxMult = settings->getEdgePanMaxMult();
         int panDir = 0;
-        if (aboveMax) {
-            panDir = 1;
-            mult = maxMult * std::min(bboxLen, bboxMax - visMax) / bboxLen;
-        } else if (belowMin) {
-            panDir = -1;
-            mult = maxMult * std::min(bboxLen, visMin - bboxMin) / bboxLen;
+
+        // If the selection is larger than the view, scroll based on mouse position relative to the center of the
+        // visible view Otherwise calculate bonus scroll amount due to proportion of selection out of view.
+        if (isLargeSelection) {
+            mult = maxMult * std::abs(mouseDiff) / (visLen);
+            if (mouseDiff > 0.1 * visLen / 2.0) {
+                panDir = 1;
+            } else if (mouseDiff < -0.1 * visLen / 2.0) {
+                panDir = -1;
+            }
+        } else {
+            if (aboveMax) {
+                panDir = 1;
+                mult = maxMult * std::min(bboxLen, bboxMax - visMax) / bboxLen;
+            } else if (belowMin) {
+                panDir = -1;
+                mult = maxMult * std::min(bboxLen, visMin - bboxMin) / bboxLen;
+            }
         }
 
         // Base amount to translate selection (in document coordinates) per timer tick
@@ -833,14 +867,15 @@ bool EditSelection::handleEdgePan(EditSelection* self) {
 
         return layoutScroll;
     };
-
     // Compute scroll (for layout) and translation (for selection) for x and y
     const int layoutWidth = layout->getMinimalWidth();
     const int layoutHeight = layout->getMinimalHeight();
     const auto visRect = layout->getVisibleRect();
     const auto bbox = self->getBoundingBoxInView();
-    const auto layoutScrollX = computeScrollAmt(visRect.x, visRect.width, bbox.x, bbox.width, layoutWidth);
-    const auto layoutScrollY = computeScrollAmt(visRect.y, visRect.height, bbox.y, bbox.height, layoutHeight);
+    const auto layoutScrollX =
+            computeScrollAmt(visRect.x, visRect.width, bbox.x, bbox.width, layoutWidth, self->relMousePosX);
+    const auto layoutScrollY =
+            computeScrollAmt(visRect.y, visRect.height, bbox.y, bbox.height, layoutHeight, self->relMousePosY);
     const auto translateX = layoutScrollX / zoom;
     const auto translateY = layoutScrollY / zoom;
 
