@@ -2,15 +2,18 @@
 
 #include <algorithm>  // for min
 #include <array>      // for array
+#include <cmath>      // for sqrt
 #include <utility>    // for move, pair
 
-#include <cairo.h>        // for cairo_surface_destroy
-#include <gdk/gdk.h>      // for gdk_cairo_set_sourc...
-#include <glib.h>         // for g_assert, guchar
+#include <cairo.h>    // for cairo_surface_destroy
+#include <gdk/gdk.h>  // for gdk_cairo_set_sourc...
+#include <glib.h>     // for g_assert, guchar
 
-#include "model/Element.h"                        // for Element, ELEMENT_IMAGE
-#include "util/Rectangle.h"                       // for Rectangle
-#include "util/raii/GObjectSPtr.h"                // for GObjectSPtr
+#include "model/Element.h"   // for Element, ELEMENT_IMAGE
+#include "util/Rectangle.h"  // for Rectangle
+#include "util/i18n.h"
+#include "util/raii/GObjectSPtr.h"  // for GObjectSPtr
+#include "util/safe_casts.h"
 #include "util/serializing/ObjectInputStream.h"   // for ObjectInputStream
 #include "util/serializing/ObjectOutputStream.h"  // for ObjectOutputStream
 
@@ -127,34 +130,79 @@ void Image::setImage(cairo_surface_t* image) {
     data = std::move(closure_.buffer);
 }
 
-auto Image::getImage() const -> cairo_surface_t* {
+auto Image::renderBuffer() const -> std::optional<std::string> {
     g_assert(data.length() > 0 && "image has no data, cannot render it!");
-    if (this->image == nullptr) {
-        xoj::util::GObjectSPtr<GdkPixbufLoader> loader(gdk_pixbuf_loader_new(), xoj::util::adopt);
-        gdk_pixbuf_loader_write(loader.get(), reinterpret_cast<const guchar*>(this->data.c_str()), this->data.length(),
-                                nullptr);
-        bool success = gdk_pixbuf_loader_close(loader.get(), nullptr);
-        g_assert(success && "errors in loading image data!");
-
-        GdkPixbuf* tmp = gdk_pixbuf_loader_get_pixbuf(loader.get());
-        g_assert(tmp != nullptr);
-        xoj::util::GObjectSPtr<GdkPixbuf> pixbuf(gdk_pixbuf_apply_embedded_orientation(tmp), xoj::util::adopt);
-
-        this->imageSize = {gdk_pixbuf_get_width(pixbuf.get()), gdk_pixbuf_get_height(pixbuf.get())};
-
-        // TODO: pass in window once this code is refactored into ImageView
-        this->image = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, this->imageSize.first, this->imageSize.second);
-        g_assert(this->image != nullptr);
-
-        // Paint the pixbuf on to the surface
-        // NOTE: we do this manually instead of using gdk_cairo_surface_create_from_pixbuf
-        // since this does not work in CLI mode.
-        cairo_t* cr = cairo_create(this->image);
-        gdk_cairo_set_source_pixbuf(cr, pixbuf.get(), 0, 0);
-        cairo_paint(cr);
-        cairo_destroy(cr);
+    if (this->image) {
+        // Already rendered
+        return std::nullopt;
+    }
+    xoj::util::GObjectSPtr<GdkPixbufLoader> loader(gdk_pixbuf_loader_new(), xoj::util::adopt);
+    g_signal_connect(loader.get(), "size-prepared",
+                     G_CALLBACK(+[](GdkPixbufLoader* self, gint width, gint height, gpointer) {
+                         static constexpr uint64_t MAX_SIZE =
+                                 1 << 25;  ///< Max number of pixels: 32M = more than enough for A4 in 72pp
+                         if (width <= 0 || height <= 0) {
+                             g_warning("Image::renderBuffer(): non-positive width/height");
+                             return;
+                         }
+                         if (static_cast<uint64_t>(width) * static_cast<uint64_t>(height) > MAX_SIZE) {
+                             double ratio = static_cast<double>(width) / static_cast<double>(height);
+                             gint maxHeight = floor_cast<gint>(std::sqrt(MAX_SIZE / ratio));
+                             gint maxWidth = floor_cast<gint>(maxHeight * ratio);
+                             g_warning("Trying to open an image too big %d x %d. Resizing it to %d x %d", width, height,
+                                       maxWidth, maxHeight);
+                             gdk_pixbuf_loader_set_size(self, maxHeight, maxWidth);
+                         }
+                     }),
+                     nullptr);
+    GError* err = nullptr;
+    bool success = gdk_pixbuf_loader_write(loader.get(), reinterpret_cast<const guchar*>(this->data.c_str()),
+                                           this->data.length(), &err);
+    if (!success) {
+        if (err != nullptr) {
+            std::string msg = std::string(_("Failed to load image")) + "\n" + _("Error: ") + err->message;
+            g_free(err);
+            return msg;
+        } else {
+            return std::string(_("Failed to load image")) + "\n" + _("Unrecoverable error");
+        }
+    }
+    success = gdk_pixbuf_loader_close(loader.get(), &err);
+    if (!success) {
+        if (err != nullptr) {
+            std::string msg = std::string(_("Failed to close image stream")) + "\n" + _("Error: ") + err->message;
+            g_free(err);
+            return msg;
+        } else {
+            return std::string(_("Failed to close image stream")) + "\n" + _("Unrecoverable error");
+        }
     }
 
+    GdkPixbuf* tmp = gdk_pixbuf_loader_get_pixbuf(loader.get());
+    g_assert(tmp != nullptr);
+    xoj::util::GObjectSPtr<GdkPixbuf> pixbuf(gdk_pixbuf_apply_embedded_orientation(tmp), xoj::util::adopt);
+
+    this->imageSize = {gdk_pixbuf_get_width(pixbuf.get()), gdk_pixbuf_get_height(pixbuf.get())};
+
+    // TODO: pass in window once this code is refactored into ImageView
+    this->image = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, this->imageSize.first, this->imageSize.second);
+    g_assert(this->image != nullptr);
+
+    // Paint the pixbuf on to the surface
+    // NOTE: we do this manually instead of using gdk_cairo_surface_create_from_pixbuf
+    // since this does not work in CLI mode.
+    cairo_t* cr = cairo_create(this->image);
+    gdk_cairo_set_source_pixbuf(cr, pixbuf.get(), 0, 0);
+    cairo_paint(cr);
+    cairo_destroy(cr);
+    return std::nullopt;
+}
+
+auto Image::getImage() const -> cairo_surface_t* {
+    if (auto opt = renderBuffer(); opt.has_value()) {
+        // An error occurred
+        g_warning("%s", opt->c_str());
+    }
     return this->image;
 }
 
