@@ -18,7 +18,7 @@ class Layer;
 template <class T>
 struct PageLayerPosEntry {
     // TODO: constructor could be removed with C++20
-    PageLayerPosEntry<T>(Layer* layer, T* element, typename T::Index pos): layer(layer), element(element), pos(pos) {}
+    PageLayerPosEntry(Layer* layer, T* element, typename T::Index pos): layer(layer), element(element), pos(pos) {}
 
     Layer* layer;
     T* element;
