@@ -48,6 +48,16 @@ auto Util::paintBackgroundWhite(GtkWidget* widget, cairo_t* cr, void*) -> gboole
     return false;
 }
 
+utl::Point<double> Util::toWidgetCoords(GtkWidget* widget, utl::Point<double> absolute_coords) {
+    int rx, ry;
+    // X11 uses absolute screen coordinates while Wayland uses absolute window coordinates.
+    // Converting them to widget-local coordinates will cancel out this difference.
+    // `gtk_widget_get_window` doesn't return the actual window, but the local widget-window.
+    // GTK4 renames `gdk_window_get_root_coords()` to `gdk_surface_get_root_coords()`
+    gdk_window_get_root_coords(gtk_widget_get_window(widget), 0, 0, &rx, &ry);
+    return utl::Point<double>{absolute_coords.x - rx, absolute_coords.y - ry};
+}
+
 void Util::cairo_set_dash_from_vector(cairo_t* cr, const std::vector<double>& dashes, double offset) {
     cairo_set_dash(cr, dashes.data(), static_cast<int>(dashes.size()), offset);
 }
